@@ -4,7 +4,7 @@ from .base import ObjectBase
 class Customer(ObjectBase):
     @classmethod
     def get_resource_class(cls, client):
-        from ..resources.customers import Customers
+        from ..resources import Customers
 
         return Customers(client)
 
@@ -42,15 +42,18 @@ class Customer(ObjectBase):
 
     @property
     def subscriptions(self):
-        """Return the subscription list for the customer."""
-        return self.client.customer_subscriptions.on(self).list()
+        from ..resources import CustomerSubscriptions
+
+        return CustomerSubscriptions(self.client, self)
 
     @property
     def mandates(self):
-        """Return the mandate list for the customer."""
-        return self.client.customer_mandates.on(self).list()
+        from ..resources import CustomerMandates
+
+        return CustomerMandates(self.client, self)
 
     @property
     def payments(self):
-        """Return the payment list for the customer."""
-        return self.client.customer_payments.on(self).list()
+        from ..resources import CustomerPayments
+
+        return CustomerPayments(self.client, self)
