@@ -1,10 +1,12 @@
+from ..resources import PaymentCaptures, PaymentChargebacks
 from .base import ObjectBase
+from .customer import Customer
 
 
 class Payment(ObjectBase):
     @classmethod
     def get_resource_class(cls, client):
-        from ..resources.payments import Payments
+        from ..resources import Payments
 
         return Payments(client)
 
@@ -142,10 +144,6 @@ class Payment(ObjectBase):
     def mandate_id(self):
         return self._get_property("mandateId")
 
-    @property
-    def subscription_id(self):
-        return self._get_property("subscriptionId")
-
     @property
     def order_id(self):
         return self._get_property("orderId")
@@ -162,56 +160,75 @@ class Payment(ObjectBase):
     def routing(self):
         return self._get_property("routing")
 
+    @property
+    def subscription_id(self):
+        return self._get_property("subscriptionId")
+
     # documented _links
 
     @property
     def checkout_url(self):
         return self._get_link("checkout")
 
+    @property
+    def changepaymentstate_url(self):
+        return self._get_link("changePaymentState")
+
+    @property
+    def payonline_url(self):
+        return self._get_link("payOnline")
+
     @property
     def refunds(self):
         """Return the refunds related to this payment."""
-        return self.client.payment_refunds.on(self).list()
+        from ..resources import PaymentRefunds
+
+        return PaymentRefunds(self.client, self)
 
     @property
     def chargebacks(self):
         """Return the chargebacks related to this payment."""
-        return self.client.payment_chargebacks.on(self).list()
+        return PaymentChargebacks(self.client, self)
 
     @property
     def captures(self):
         """Return the captures related to this payment"""
-        return self.client.captures.on(self).list()
+        return PaymentCaptures(self.client, self)
 
-    @property
-    def settlement(self):
+    def get_settlement(self):
         """Return the settlement for this payment."""
-        return self.client.settlements.get(self.settlement_id)
+        if self.settlement_id:
+            return self.client.settlements.get(self.settlement_id)
 
-    @property
-    def mandate(self):
+    def get_mandate(self):
         """Return the mandate for this payment."""
-        return self.client.customer_mandates.with_parent_id(self.customer_id).get(self.mandate_id)
+        if self.customer_id and self.mandate_id:
+            # Setup a minimal Customer object without querying the API.
+            customer = Customer({"id": self.customer_id}, self.client)
+            return customer.mandates.get(self.mandate_id)
 
-    @property
-    def subscription(self):
-        """Return the subscription for this payment."""
-        return self.client.customer_subscriptions.with_parent_id(self.customer_id).get(self.subscription_id)
+    def get_subscription(self):
+        """
+        Return the subscription for this payment.
 
-    @property
-    def customer(self):
+        This is only available for recurring payments.
+        """
+        url = self._get_link("subscription")
+        if url:
+            from ..resources import CustomerSubscriptions
+
+            return CustomerSubscriptions(self.client, customer=None).from_url(url)
+
+    def get_customer(self):
         """Return the customer for this payment."""
-        return self.client.customers.get(self.customer_id)
+        if self.customer_id:
+            return self.client.customers.get(self.customer_id)
 
-    @property
-    def order(self):
+    def get_order(self):
         """Return the order for this payment."""
-        from ..resources.orders import Order
-
         url = self._get_link("order")
         if url:
-            resp = self.client.orders.perform_api_call(self.client.orders.REST_READ, url)
-            return Order(resp, self.client)
+            return self.client.orders.from_url(url)
 
     # additional methods
 
