@@ -1,10 +1,10 @@
-from .base import Base
+from .base import ObjectBase
 
 
-class Permission(Base):
+class Permission(ObjectBase):
     @classmethod
     def get_resource_class(cls, client):
-        from ..resources.permissions import Permissions
+        from ..resources import Permissions
 
         return Permissions(client)
 
