@@ -1,3 +1,5 @@
+import re
+
 from .base import ObjectBase
 from .customer import Customer
 
@@ -5,7 +7,7 @@ from .customer import Customer
 class Subscription(ObjectBase):
     @classmethod
     def get_resource_class(cls, client):
-        from ..resources.customer_subscriptions import CustomerSubscriptions
+        from ..resources import CustomerSubscriptions
 
         return CustomerSubscriptions(client)
 
@@ -58,6 +60,10 @@ class Subscription(ObjectBase):
     def times(self):
         return int(self._get_property("times"))
 
+    @property
+    def times_remaining(self):
+        return int(self._get_property("timesRemaining"))
+
     @property
     def interval(self):
         return self._get_property("interval")
@@ -66,6 +72,10 @@ class Subscription(ObjectBase):
     def start_date(self):
         return self._get_property("startDate")
 
+    @property
+    def next_payment_date(self):
+        return self._get_property("nextPaymentDate")
+
     @property
     def description(self):
         return self._get_property("description")
@@ -74,6 +84,10 @@ class Subscription(ObjectBase):
     def method(self):
         return self._get_property("method")
 
+    @property
+    def mandate_id(self):
+        return self._get_property("mandateId")
+
     @property
     def canceled_at(self):
         return self._get_property("canceledAt")
@@ -87,15 +101,47 @@ class Subscription(ObjectBase):
         return self._get_property("metadata")
 
     @property
-    def customer(self):
+    def application_fee(self):
+        return self._get_property("applicationFee")
+
+    def get_customer(self):
         """Return the customer for this subscription."""
         url = self._get_link("customer")
-        if url:
-            resp = self.client.customers.perform_api_call(self.client.customers.REST_READ, url)
-            return Customer(resp)
+        return self.client.customers.from_url(url)
+
+    @property
+    def customer_id(self):
+        """
+        Retrieve the customer id from the customer link.
+
+        The customer_id is not available as a direct subscription property,
+        but we need it to implement various features that need it. The only
+        option is to extract it from the link.
+        """
+        url = self._get_link("customer")
+        matches = re.findall(r"/customers/(cst_\w+)", url)
+        if matches:
+            return matches[0]
+
+    def get_profile(self):
+        """Return the profile related to this subscription."""
+        url = self._get_link("profile")
+        if not url:
+            return None
+        return self.client.profiles.from_url(url)
+
+    def get_mandate(self):
+        if self.mandate_id and self.customer_id:
+            from ..resources import CustomerMandates
+
+            customer = Customer({"id": self.customer_id}, self.client)
+            return CustomerMandates(self.client, customer).get(self.mandate_id)
 
     @property
     def payments(self):
-        """Return a list of payments for this subscription."""
-        payments = self.client.subscription_payments.on(self).list()
-        return payments
+        # We could also have implemented this using the "payments" entry from the _links, but then we would not have
+        # the explicit interface using .payments.list()
+        from ..resources import SubscriptionPayments
+
+        customer = Customer({"id": self.customer_id}, self.client)
+        return SubscriptionPayments(self.client, customer=customer, subscription=self)
